<?php

# Copyright (c)  2008 - Marcus Lunzenauer <mlunzena@uos.de>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


/**
 * <ClassDescription>
 *
 * @package     <package>
 * @subpackage  <package>
 *
 * @author    mlunzena
 * @copyright (c) Authors
 * @version   $Id: trac.php 618 2008-04-03 09:50:54Z mlunzena $
 */

class Trac {


  private $xmlrpc_client;


  protected function __construct($url, $username, $password, $debug = 0) {
    $this->xmlrpc_client =& new xmlrpc_client($url);
    $this->xmlrpc_client->setCredentials($username, $password);
    $this->xmlrpc_client->setDebug($debug);
    $this->xmlrpc_client->return_type = 'phpvals';
  }


  static function connect($url = NULL, $username = NULL, $password = NULL,
                         $debug = 0) {

    static $connection;

    if (isset($url)) {
      $connection = new Trac($url, $username, $password);
    }

    return $connection;
  }


  function send($msgs) {

    $single_message = !is_array($msgs);

    $map = create_function('$a',
      'return php_xmlrpc_encode($a, array("auto_dates"));');

    if ($single_message) {

      $msg = func_get_args();

      $msg0 = array_shift($msg);
      $args = array_map($map, $msg);
      $xmlrpc_msg = new xmlrpcmsg($msg0, $args);

      $response = $this->xmlrpc_client->send($xmlrpc_msg);

      if ($code = $response->faultCode()) {
        throw new Trac_Exception($response->faultString(), $code);
      }

      return $response->value();

    }

    else {

      $xmlrpc_msgs = array();
      foreach ($msgs as $msg) {
        $msg0 = array_shift($msg);
        $args = array_map($map, $msg);
        $xmlrpc_msgs[] = new xmlrpcmsg($msg0, $args);
      }

      $responses = $this->xmlrpc_client->send($xmlrpc_msgs);

      $results = array();
      foreach ($responses as $response) {
        if ($code = $response->faultCode())
          throw new Trac_Exception($response->faultString(), $code);
        else
          $results[] = $response->value();
      }

      return $results;
    }
  }
}
